@@ -1,3 +1,29 @@
+##  Archive::Unrar
+
+##  Copyright (C) 2009 Nikos Vaggalis <nikos.vaggalis@gmail.com>
+##  This program is free software; you can redistribute it and/or modify
+##  it under the terms of the GNU General Public License as published by
+##  the Free Software Foundation; either version 3 of the License, or
+##  (at your option) any later version.
+
+##  This program is distributed in the hope that it will be useful,
+##  but WITHOUT ANY WARRANTY; without even the implied warranty of
+##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+##  GNU General Public License for more details.
+
+##  You should have received a copy of the GNU General Public License
+##  along with this program; if not, write to the Free Software
+##  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
+######## Archive::Unrar library  ########
+###
+#Author  : Nikos Vaggalis
+#Email   : nikos.vaggalis@gmail.com
+#Version Date: 10-12-2009
+#License :GNU General Public License (GPL) v3
+#Copyright 2009 Nikos Vaggalis
+#########
+
 package Archive::Unrar;
 
 use 5.010000;
@@ -11,12 +37,12 @@ use constant COMMENTS_BUFFER_SIZE => 16384;
 
 our @EXPORT_OK = qw(list_files_in_archive process_file);
 
-our $VERSION = '1.00';
+our $VERSION = '1.1';
 
 #unrar.dll internal functions
 our (
     $RAROpenArchiveEx, $RARCloseArchive, $RAROpenArchive, $RARReadHeader,
-    $RARReadHeaderEx,  $RARProcessFile,  $RARSetPassword
+    $RARReadHeaderEx,  $RARProcessFile,  $RARSetPassword, %donotprocess
 );
 
 ################ PRIVATE METHODS ################
@@ -49,7 +75,7 @@ sub extract_headers {
     my $RARHeaderData = pack( 'x260x260LLLLLLLLLLpLL',
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, undef, 0, 0 );
 
-    my $handle1 = $RAROpenArchiveEx->Call($RAROpenArchiveDataEx)
+    my $handle1 = $RAROpenArchiveEx->Win32::API::Call($RAROpenArchiveDataEx)
       || croak "RAROpenArchiveEx failed";
 
     my (
@@ -90,6 +116,9 @@ sub extract_headers {
     if ( !( $flagsEX & 128 ) && !( $flagsEX & 256 ) && ( $flagsEX & 1 ) ) {
         $continue = "no";
     }
+    elsif ( exists $donotprocess{$file} ) {
+        $continue = "no";
+    }
 
     !$RARCloseArchive->Call($handle2) || croak "RARCloseArchive failed";
     return ( $flagsEX & 128, $flags & 4, $continue );
@@ -128,6 +157,7 @@ sub list_files_in_archive {
         if ( $processresult != 0 ) {
             $errorcode =
               "Error from dll - (Check dll documentation): " . $processresult;
+
 #probably wrong password but check unrar.dll documentation for error description
             last;
         }
@@ -136,6 +166,7 @@ sub list_files_in_archive {
             print "File\t\t\t\t\tSize\n";
             print "-------------------------------------------\n";
             print "$files[0]\\$files[1]\t\t$files[4]\n\n";
+            $donotprocess{ $files[0] } = 1 if ($blockencrypted);
         }
 
     }
@@ -149,16 +180,21 @@ sub list_files_in_archive {
 }
 
 sub process_file {
-    my ( $file, $password ) = @_;
+    my ($file, $password, $output_dir_path)=@_;
     my ( $blockencrypted, $needpassword, $continue ) = extract_headers($file);
     my $errorcode;
 
     return ( $errorcode = "multipart" ) if ($continue);
 
     my $directory;
-    ( $directory = $file ) =~ s/\.rar//i;
+    ( $directory = $file ) =~ s/\.rar$//i;
 
-    my $blockencryptedflag;
+	if (defined($output_dir_path)) {
+	    $directory=~s/.*\\//;
+		$directory=$output_dir_path."\\".$directory;
+	}
+    
+	my $blockencryptedflag;
 
     my $RAROpenArchiveDataEx_for_extracting =
       pack( 'ppLLpLLLLx32', $file, undef, 1, 0, undef, 0, 0, 0, 0 );
@@ -185,16 +221,21 @@ sub process_file {
           $RARProcessFile->Call( $handle, 2, $directory, undef );
         if ( $processresult != 0 ) {
             $errorcode =
-              "Error from dll - (Check dll documentation): " . $processresult;
+              "Error from dll - (Check dll documentation): " . $processresult;          
 #probably wrong password but check unrar.dll documentation for error description
             last;
         }
 
     }
 
+	
+	
     if ( $blockencrypted && ( !defined($blockencryptedflag) ) ) {
         $errorcode = "headers encrypted and password not correct";
     }
+    elsif ($blockencrypted) {
+        list_files_in_archive( $file, $password );
+    }    
 
     !$RARCloseArchive->Call($handle) || croak "RRARCloseArchive failed";
     return $errorcode;
@@ -213,17 +254,29 @@ Archive::Unrar - is a procedural module that provides manipulation (extraction a
 
 	use Archive::Unrar qw(list_files_in_archive process_file);
 	
-	#usage
-	list_files_in_archive($file,$password);
-	process_file($file,$password); 
+	#usage with password
+		list_files_in_archive($file,$password);
+		process_file($file,$password); 
 	
+	#usage without password
+		list_files_in_archive($file,undef);
+		process_file($file,undef); 
+		
 	#if RAR archive in the same directory as the caller
-	list_files_in_archive("myfile.rar","mypassword");
-	process_file("myfile.rar","mypassword"); 
+		list_files_in_archive("testwithpass.rar","mypassword");
+		process_file("testwithpass.rar","mypassword"); 
 	
-	#absolute path if RAR archive not in the same directory as the caller
-	list_files_in_archive("c:\mydirectory\myfile.rar","mypassword");
-	process_file("c:\mydirectory\myfile.rar","mypassword"); 
+	#absolute path if RAR archive is not in the same directory as the caller
+		list_files_in_archive("c:\\input_dir\\testwithpass.rar","mypassword");
+		process_file("c:\\input_dir\\testwithpass.rar","mypassword"); 
+	
+	#optionally, provide output directory as the last parameter,
+	#if directory does not exist then it will be automatically created
+	#if output directory is not provided then the file is extracted 
+	#in the same directory the caller
+		list_files_in_archive("c:\\input_dir\\testwithpass.rar","mypassword");
+		process_file("c:\\input_dir\\testwithpass.rar","mypassword","c:\\output_dir"); 
+		process_file("c:\\input_dir\\testnopass.rar",undef,"c:\\output_dir"); 
 		
 
 =head1 DESCRIPTION
@@ -232,11 +285,13 @@ Archive::Unrar is a procedural module that provides manipulation (extraction and
 
 It uses two functions : list_files_in_archive and process file
 
-The first one lists details embedded into the archive (files bundled into the .rar archive,archive's comments and header info) and the latter extracts the files from the archive. The extraction process generates a directory which matches the naming of
-the archive, making identification easy;for example an archive named 'myarchive.rar' will create a directory called 'myarchive'
-and inside that directory will the contents of arhive be extracted.
+The first one lists details embedded into the archive (files bundled into the .rar archive,archive's comments and header info) and the latter extracts the files from the archive.
 
-Both take two parameters;the first is the file name and the second is the password required by the archive.
+list_files_in_archive takes two parameters;the first is the file name and the second is the password required by the archive.
+If no password is required then just pass undef or the empty string as the second parameter
+
+process_file takes three parameters;the first is the file name, the second is the password required by the archive
+and the third is the directory that the file's contents will be extracted to.
 If no password is required then just pass undef or the empty string as the second parameter
 
 Both procedures return undef if successfull, and an error description if something went wrong
@@ -255,10 +310,10 @@ This package includes the dll,samples,dll internals and error description
 =head1 TEST AFTER INSTALLATION
 
 After module is installed run test\mytest.pl.
-If all is well then you should see two files inside two directories :
+If all is well then you should see two files in the directory :
 
-	testnopass\test no pass succedeed.txt
-	testwithpass\test with pass succedeed.txt
+	test no pass succedeed.txt
+	test with pass succedeed.txt
 
 =head2 EXPORT
 
@@ -268,13 +323,9 @@ None by default.
 
 Nikos Vaggalis <F<nikos.vaggalis@gmail.com>>
 
-=head1 COPYRIGHT AND LICENSE
-
-Copyright (C) 2009 by Nikos Vaggalis
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.10.0 or,
-at your option, any later version of Perl 5 you may have available.
+For a complete application based on the module look at :
+L<http://sourceforge.net/projects/unrarextractrec/>
 
+Licence : GNU General Public License 3.0
 
 =cut
